from django.core.management.base import BaseCommand

from chart.models import Chart

class Command(BaseCommand):
    def handle(self, *app_labels, **options):
        object_list = Chart.objects.all()
        for obj in object_list:
            for entry in obj.chartentries.all():
                if entry.remove:
                    entry.delete()
                else:
                    entry.previous_position = entry.current_position
                    entry.current_position = entry.next_position
                    entry.next_position = 0
                    entry.save()



from django import template

register = template.Library()

@register.inclusion_tag('chart/inclusion_tags/chartentry_listing.html')
def chartentry_listing(object_list):
    return {'object_list': object_list}



import random

from generate import IMAGES
from generate.json_loader import load_json

CHART_COUNT = 1
CHART_ENTRY_COUNT = 40
CREDIT_COUNT = 40
TRACK_COUNT = 20

CONTRIBUTOR_ROLE = [[1, "Artist"], [2, "Producer"], [3, "Composer"]]

def generate():
    objects = []
    
    # generate chart track objects
    for i in range(1, CREDIT_COUNT + 1):
        contributor_role = random.choice(CONTRIBUTOR_ROLE)
        objects.append({
            "model": "music.Credit",
            "fields": {
                "role": contributor_role[0],
                "track": {
                    "model": "music.Track",
                    "fields": {
                        "title": "Chart Track %s Title" % i,
                        "description": "Chart Track %s description with some added text to verify truncates where needed." % i,
                        "state": "published",
                        "image": random.sample(IMAGES, 1)[0],
                        "video_embed": "",
                        "sites": {
                            "model": "sites.Site",
                            "fields": { 
                                "name": "example.com"
                            }
                        },
                        "album": {
                            "model": "music.Album",
                            "fields": {
                                "title": "Chart Album %s Title" % i,
                                "description": "Chart Album %s description with some added text to verify truncates where needed." % i,
                                "state": "published",
                                "image": random.sample(IMAGES, 1)[0],
                                "sites": {
                                    "model": "sites.Site",
                                    "fields": { 
                                        "name": "example.com"
                                    }
                                },
                            },
                        },
                    },
                },
                "contributor": {
                    "model": "music.TrackContributor",
                    "fields": {
                        "title": "Chart Track Contributor %s Title" % i,
                        "description": "Chart Track Contributor %s description with some added text to verify truncates where needed." % i,
                        "state": "published",
                        "image": random.sample(IMAGES, 1)[0],
                        "sites": {
                            "model": "sites.Site",
                            "fields": { 
                                "name": "example.com"
                            }
                        },
                    },
                },
            },
        })
    
    # gen chart entry objects
    for i in range(1, CHART_ENTRY_COUNT + 1):
        
        objects.append({
            "model": "chart.ChartEntry",
            "fields": {
                "previous_position": random.randint(1, 40),
                "current_position": i,
                "next_position": random.randint(1, 40),
                "remove": False,
                "chart": {
                    "model": "chart.Chart",
                    "fields": {
                        "title": "Chart 1 Title",
                        "state": "published",
                        "image": random.sample(IMAGES, 1)[0],
                        "sites": {
                            "model": "sites.Site",
                            "fields": { 
                                "name": "example.com"
                            }
                        },
                    }
                },
                "track": {
                    "model": "music.Track",
                    "fields": {
                        "title": "Chart Track %s Title" % i,
                    }
                },
            },
        })
    
    # create chart options
    objects.append({
        "model": "preferences.ChartPreferences",
        "fields": {
            "primary_chart": {
                "model": "chart.Chart",
                "fields": {
                    "title": "Chart 1 Title",
                }
            },
        }
    })
    
    load_json(objects)



from jmbo.generic.views import GenericObjectDetail, GenericObjectList
from jmbo.view_modifiers import IntegerFieldRangeViewModifier
from chart.models import Chart
from jmbo.view_modifiers import IntegerFieldRangeViewModifier

class ObjectList(GenericObjectList):
    def get_extra_context(self, *args, **kwargs):
        extra_context = super(ObjectList, self).get_extra_context(*args, **kwargs)
        added_context = {'title': 'Charts'}
        if extra_context:
            extra_context.update(
                added_context,
            )
        else:
            extra_context = added_context

        return extra_context
    
    def get_paginate_by(self):
        return 12
    
    def get_queryset(self):
        return Chart.permitted.all()

object_list = ObjectList()

class ObjectDetail(GenericObjectList):
    def get_queryset(self, *args, **kwargs):
        slug = kwargs['slug']
        return Chart.permitted.get(slug=slug).chartentries.all().order_by('current_position')
   
    def get_view_modifier(self, request, *args, **kwargs):
        return IntegerFieldRangeViewModifier(request=request, field_name="current_position", count=self.get_queryset(*args, **kwargs).count(), interval=10)
    
object_detail = ObjectDetail()



from django.contrib import admin

from jmbo.admin import ModelBaseAdmin
from chart.models import Chart, ChartEntry, ChartPreferences


class ChartEntryAdmin(admin.ModelAdmin):
    list_display = ('chart', 'track', 'current_position', 'remove')
    list_filter = ('chart', 'created')
    search_fields = ('created',)

admin.site.register(Chart, ModelBaseAdmin)
admin.site.register(ChartEntry, ChartEntryAdmin)
admin.site.register(ChartPreferences)



from datetime import datetime

from django.core.urlresolvers import reverse
from django.db import models

from jmbo.models import ModelBase
from music.models import Track
from preferences.models import Preferences

class Chart(ModelBase):
    class Meta:
        verbose_name = 'Chart'
        verbose_name_plural = 'Charts'
    
    def get_absolute_url(self):
        return reverse('chart_object_detail', kwargs={'slug': self.slug})
        
    def __unicode__(self):
        return self.title

class ChartEntry(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    chart = models.ForeignKey(
        Chart, 
        related_name='chartentries'
    )
    track = models.ForeignKey(Track)
    previous_position = models.IntegerField(
        blank=True,
        null=True
    )
    current_position = models.IntegerField(
        blank=True,
        null=True
    )
    next_position = models.IntegerField(
        blank=True,
        null=True
    )
    remove = models.BooleanField(
        help_text="On the next update this entry will be removed completely."
    )
    
    class Meta:
        verbose_name = 'Chart entry'
        verbose_name_plural = 'Chart entries'
        ordering = ['current_position']
    
    def get_duration_on_chart(self):
        return datetime.now() - (datetime.now() - self.created)
    
    def __unicode__(self):
        return '%s Entry %s' % (self.chart.title, self.current_position)
        
class ChartPreferences(Preferences):
    __module__ = 'preferences.models'

    primary_chart = models.ForeignKey(
        'chart.Chart',
        null=True,
        help_text="Select the primary chart link from the navigation.",
        related_name='chartoptions_primary_chart',
        limit_choices_to = {'state': 'published'}
    )
    
    class Meta:
        verbose_name = 'Chart preferences'
        verbose_name_plural = 'Chart preferences'
